import unittest

import numpy as np

from audiodag.signal.components.tonal_component import SineComponent
from tests.unit.signal.components.test_component import SHOW


class TestSineComponent(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls._sut = SineComponent(fs=10000, duration=20, mag=1, clip=np.inf)

    def test_plot(self):
        self._sut.plot(show=SHOW)

    def test_normalise_limits_range(self):
        ev_1 = SineComponent(fs=10000, normalise=False)
        ev_2 = SineComponent(fs=10000, normalise=True)

        self.assertFalse(ev_1 == ev_2)
        self.assertFalse(np.all(ev_1.y == ev_2.y))
        self.assertLessEqual(ev_2.y.max(), 1.0)
        self.assertGreaterEqual(ev_2.y.min(), 0.0)

    def test_cache_keeps_generated_array(self):
        # Arrange
        ev1 = SineComponent(fs=200, duration=20, mag=1, clip=10.0, cache=False)
        ev2 = SineComponent(fs=200, duration=20, mag=1, clip=10.0, cache=True)

        # Act
        ev1_y = ev1.y
        ev2_y = ev2.y

        # Assert
        self.assertIsInstance(ev1_y, np.ndarray)
        self.assertIsInstance(ev2_y, np.ndarray)
        self.assertIsNone(ev1._y)
        self.assertIsNotNone(ev2._y)



import unittest

import numpy as np

from audiodag.signal.envelopes.templates import ConstantEnvelope, CosEnvelope, CosRiseEnvelope

# Use to show plots when debugging
SHOW = False

ONES = np.ones(shape=(1000,))
ZEROS = np.zeros(shape=(800,))


class TestConstantEnvelope(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls._sut = ConstantEnvelope()

    def test_envelope_no_effect_on_energy_in_constant_signal(self) -> None:
        self.assertTrue(np.all(self._sut(ONES) == ONES))

    def test_envelope_no_effect_on_energy_in_empty_signal(self) -> None:
        self.assertTrue(np.all(self._sut(ZEROS) == ZEROS))

    def test_eval_repr_equality(self):
        clone = eval(self._sut.__repr__())
        self.assertEqual(self._sut, clone)


class TestCosEnvelope(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls._sut = CosEnvelope()

    def test_envelope_reduces_energy_of_constant_signal(self) -> None:
        self.assertLess(np.sum(self._sut(ONES)), np.sum(ONES))

    def test_envelope_no_effect_on_energy_in_empty_signal(self) -> None:
        self.assertTrue(np.all(np.round(self._sut(ZEROS)) == ZEROS))

    def test_eval_repr_equality(self):
        clone = eval(self._sut.__repr__())
        self.assertEqual(self._sut, clone)


class TestCosRiseEnvelope(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls._sut = CosRiseEnvelope(fs=1000,
                                   rise=200)

    def test_envelope_reduces_energy_of_constant_signal(self) -> None:
        self.assertLess(np.sum(self._sut(ONES)), np.sum(ONES))

    def test_envelope_no_effect_on_energy_in_empty_signal(self) -> None:
        self.assertTrue(np.all(self._sut(ZEROS) == ZEROS))

    def test_eval_repr_equality(self):
        clone = eval(self._sut.__repr__())
        self.assertEqual(self._sut, clone)



import unittest
from unittest.mock import patch

import numpy as np

from audiodag.signal.digital.conversion import pts_to_ms, ms_to_pts
from audiodag.signal.digital.digital_siginal import DigitalSignal
from unittest.mock import MagicMock

# Use to show plots when debugging
SHOW = False


class TestFunctions(unittest.TestCase):
    def test_ms_to_pts(self):
        self.assertEqual(ms_to_pts(t_ms=1000,
                                   fs=1000), 1000)

        self.assertEqual(ms_to_pts(t_ms=500,
                                   fs=1000), 500)

        self.assertEqual(ms_to_pts(t_ms=1000,
                                   fs=500), 500)

        self.assertEqual(ms_to_pts(t_ms=20,
                                   fs=800), 16)

    def test_pts_to_ms(self):
        self.assertEqual(pts_to_ms(t_pts=1000,
                                   fs=1000), 1000)

        self.assertEqual(pts_to_ms(t_pts=500,
                                   fs=1000), 500)


class TestDigitalSignal(unittest.TestCase):

    @classmethod
    @patch.multiple(DigitalSignal, __abstractmethods__=set())
    def setUpClass(cls) -> None:
        cls._sut_1 = DigitalSignal(fs=100,
                                   duration=1000)
        cls._sut_2 = DigitalSignal(fs=100,
                                   duration=500)

    def test_equality_between_equals(self):
        self.assertEqual(self._sut_1, self._sut_1)

    def test_equality_between_unequals(self):
        self.assertNotEqual(self._sut_1, self._sut_2)

    def test_unique_works_as_expected(self):
        self.assertEqual(len(np.unique([self._sut_1, self._sut_1])), 1)
        self.assertEqual(len(np.unique([self._sut_1, self._sut_2])), 2)

    def test_duration_in_pts_property_converts_as_expected(self):
        self.assertEqual(self._sut_1.duration_pts, 100)
        self.assertEqual(self._sut_2.duration_pts, 50)

    @patch.multiple(DigitalSignal, __abstractmethods__=set())
    def test_init_with_defaults_sets_correct_duration(self):
        dt = DigitalSignal()

        self.assertEqual(len(dt.x), 20)
        self.assertEqual(len(dt.x_pts), 20)

    def test_no_exceptions_on_plot_call(self):
        self._sut_1.plot(show=SHOW)

    @patch.multiple(DigitalSignal, __abstractmethods__=set())
    def test_eval_repr_equality(self):
        """Note that __repr__ is currently used to define equality for DigitalSignal, making this test currently
        somewhat pointless, but this may change in the future."""

        # Used by eval
        # noinspection PyPep8Naming
        ConstantEnvelope = MagicMock()

        clone = eval(self._sut_1.__repr__())
        self.assertEqual(self._sut_1, clone)



import os
import unittest

from audiodag.signal.components.component import CompoundComponent, Component
from audiodag.signal.components.noise_component import NoiseComponent
from audiodag.signal.components.tonal_component import SineComponent

# Use to show plots when debugging
SHOW = os.environ.get('SHOW_DEBUG_PLOTS') is not None


class TestCompoundComponent(unittest.TestCase):
    _sut = CompoundComponent

    def test_compound_plus_normal_event(self):
        # Arrange
        ev_kwargs = {'fs': 1500, 'duration': 100}
        n = 3
        start_step = 80
        evs = [SineComponent(freq=10, start=s_i * start_step, **ev_kwargs) for s_i in range(n)]
        compound_event_1 = CompoundComponent(evs)
        noise_event = NoiseComponent(mag=0.06, fs=ev_kwargs['fs'], duration=400)

        # Act
        compound_event_2 = CompoundComponent([compound_event_1, noise_event])

        # Assert
        self.assertEqual(600, compound_event_2.y.shape[0])
        self.assertEqual(600, compound_event_2.channels().shape[1])
        self.assertEqual(600, compound_event_2.duration_pts)
        self.assertEqual(400, compound_event_2.duration)
        self.assertEqual(2, compound_event_2.channels().shape[0])
        self.assertEqual(2, len(compound_event_2.events))
        self.assertEqual(1500, compound_event_2.fs)
        self.assertEqual(0, compound_event_2.start)
        self.assertEqual(0, compound_event_2.start_pts)

        compound_event_2.plot(show=SHOW, channels=True)
        compound_event_1.plot_subplots(show=SHOW)
        compound_event_2.plot_subplots(show=SHOW)

    def test_compound_plus_compound_event(self):
        # Arrange
        ev_kwargs = {'fs': 1600, 'duration': 200}
        n = 3
        start_step = 80
        evs = [SineComponent(freq=10, start=s_i * start_step, **ev_kwargs) for s_i in range(n)]
        compound_event_1 = CompoundComponent(evs)
        noise_events = [NoiseComponent(mag=0.1, start=100, **ev_kwargs),
                        NoiseComponent(mag=0.06, start=300, **ev_kwargs)]
        compound_event_2 = CompoundComponent(noise_events)

        # Act
        compound_event_3 = self._sut([compound_event_1, compound_event_2])

        # Assert
        self.assertEqual(640, compound_event_2.y.shape[0])
        self.assertEqual(640, compound_event_2.channels().shape[1])
        self.assertEqual(640, compound_event_2.duration_pts)
        self.assertEqual(400, compound_event_2.duration)
        self.assertEqual(2, compound_event_2.channels().shape[0])
        self.assertEqual(2, len(compound_event_2.events))
        self.assertEqual(1600, compound_event_2.fs)
        self.assertEqual(100, compound_event_2.start)
        self.assertEqual(160, compound_event_2.start_pts)

        compound_event_1.plot_subplots(show=SHOW)
        compound_event_2.plot(show=SHOW, channels=True)
        compound_event_2.plot_subplots(show=SHOW)
        compound_event_3.plot(show=SHOW, channels=True)
        compound_event_3.plot_subplots(show=SHOW)

    def test_eval_or_repr_equal_for_complex_event(self):
        # Arrange
        ev_kwargs = {'fs': 1600, 'duration': 200}
        n = 3
        start_step = 80
        evs = [SineComponent(freq=10, start=s_i * start_step, **ev_kwargs) for s_i in range(n)]
        compound_event_1 = CompoundComponent(evs)
        noise_events = [NoiseComponent(mag=0.1, start=100, **ev_kwargs),
                        NoiseComponent(mag=0.06, start=300, **ev_kwargs)]
        compound_event_2 = CompoundComponent(noise_events)
        compound_event_3 = CompoundComponent([compound_event_1, compound_event_2])

        # Act
        clone = eval(compound_event_3.__repr__())

        # Assert
        self.assertEqual(compound_event_3, clone)
        self.assertListEqual(compound_event_3.events, clone.events)
        self.assertEqual(clone.channels().shape[0], compound_event_3.channels().shape[0])
        self.assertEqual(clone.channels().shape[1], compound_event_3.channels().shape[1])
        self.assertEqual(clone.fs, compound_event_3.fs)
        self.assertEqual(len(clone.events), len(compound_event_3.events))
        self.assertListEqual(clone.events, compound_event_3.events)
        self.assertEqual(clone.duration, compound_event_3.duration)
        self.assertEqual(clone.duration_pts, compound_event_3.duration_pts)
        self.assertEqual(clone.start, compound_event_3.start)
        self.assertEqual(clone.start_pts, compound_event_3.start_pts)

    def test_specific_msi_models_two_gap_bug_working_condition(self):
        """Testing specific bug from use in msi models stim."""
        stim = CompoundComponent(events=[
            Component(start=0, duration=2000, mag=0.02, fs=1000, seed=1283778829, cache=False, clip=2.0,
                      weight=0.08695652173913043),
            Component(start=1205, duration=50, mag=0.02, fs=1000, seed=604704893, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1255, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1255, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1280, duration=25, mag=0.02, fs=1000, seed=265799168, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1305, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1305, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1330, duration=50, mag=0.02, fs=1000, seed=335768414, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1380, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1380, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1405, duration=50, mag=0.02, fs=1000, seed=1914910913, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1455, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1455, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1480, duration=25, mag=0.02, fs=1000, seed=98974688, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1505, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1505, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1530, duration=25, mag=0.02, fs=1000, seed=1642792235, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1555, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1555, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1580, duration=50, mag=0.02, fs=1000, seed=246687274, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1630, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1630, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1655, duration=50, mag=0.02, fs=1000, seed=449561071, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1705, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1705, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1730, duration=25, mag=0.02, fs=1000, seed=1339302259, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1755, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1755, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1780, duration=50, mag=0.02, fs=1000, seed=700116608, cache=False, clip=2.0,
                      weight=0.043478260869565216), CompoundComponent(events=[
                Component(start=1830, duration=25, mag=2, fs=1000, seed=885587907, cache=False, clip=2.0, weight=0.5),
                Component(start=1830, duration=25, mag=0.02, fs=1000, seed=1242102769, cache=False, clip=2.0,
                          weight=0.5)]),
            Component(start=1855, duration=50, mag=0.02, fs=1000, seed=850417960, cache=False, clip=2.0,
                      weight=0.043478260869565216)])

        self.assertEqual(2000, len(stim.y))



import unittest
from functools import partial
from unittest.mock import MagicMock

import numpy as np

from audiodag.signal.components.component import CompoundComponent
from audiodag.signal.components.noise_component import NoiseComponent
from audiodag.signal.components.tonal_component import SineComponent
from audiodag.signal.envelopes.templates import CosRiseEnvelope
from tests.unit.signal.components.test_component import SHOW


class TestCompoundComponent(unittest.TestCase):
    _sut = CompoundComponent

    @staticmethod
    def _mock_compound_event(fs, start, duration_pts) -> MagicMock:
        return MagicMock(fs=fs,
                         start=start,
                         duration_pts=duration_pts,
                         x=np.linspace(start, duration_pts, duration_pts))

    def test_sine_noise_multiply(self):
        # Arrange
        sine_event = SineComponent(fs=800, duration=20)
        noise_event = NoiseComponent(mag=0.3, fs=800, duration=20)

        # Act
        compound_event = sine_event * noise_event

        # Assert
        self.assertEqual(20, compound_event.duration)
        self.assertEqual(16, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)
        self.assertEqual(noise_event.fs, compound_event.fs)
        self.assertEqual(sine_event.fs, compound_event.fs)

    def test_construct_from_list_of_2(self):
        # Arrange
        sine_event = SineComponent()

        # Act
        compound_event = self._sut(events=[sine_event, sine_event])

        # Assert
        self.assertEqual(1000, compound_event.fs)
        self.assertEqual(20, compound_event.duration)
        self.assertEqual(20, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)
        self.assertEqual(2, len(compound_event.events))
        self.assertAlmostEqual(float(np.mean(sine_event.y)), float(np.mean(compound_event.y)), 0)
        # This should be same as weighting is equal proportions by default
        self.assertAlmostEqual(float(np.std(compound_event.y)), float(np.std(sine_event.y)))

    def test_construct_from_list_of_2_set_weights_that_sum_to_one(self):
        # Arrange
        sine_event = SineComponent(weight=0.5, seed=123)
        sine_event_2 = SineComponent(weight=0.5, seed=231)

        # Act
        compound_event = self._sut(events=[sine_event, sine_event_2])

        # Assert
        self.assertEqual(1000, compound_event.fs)
        self.assertEqual(20, compound_event.duration)
        self.assertEqual(20, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)
        self.assertEqual(2, len(compound_event.events))
        self.assertAlmostEqual(float(np.mean(sine_event.y)), float(np.mean(compound_event.y)), 0)
        self.assertAlmostEqual(compound_event.events[0].weight, 0.5)
        self.assertAlmostEqual(compound_event.events[1].weight, 0.5)

    def test_construct_from_list_of_2_set_weights_that_dont_sum_to_one(self):
        sine_event = SineComponent(weight=1.0, seed=123)
        sine_event_2 = SineComponent(weight=0.5, seed=231)

        compound_event = self._sut(events=[sine_event, sine_event_2])

        # Assert
        # Check weight of original objects is unchanged but compound is normalised
        self.assertAlmostEqual(sine_event.weight, 1.0, 3)
        self.assertAlmostEqual(sine_event_2.weight, 0.5, 3)
        self.assertAlmostEqual(compound_event.events[0].weight, 0.667, 3)
        self.assertAlmostEqual(compound_event.events[1].weight, 0.333, 3)
        self.assertEqual(2, len(compound_event.events))
        self.assertEqual(1000, compound_event.fs)
        self.assertEqual(20, compound_event.duration)
        self.assertEqual(20, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)

    def test_construct_from_list_of_3(self):
        # Arrange
        sine_event = SineComponent()
        noise_event = NoiseComponent()

        # Act
        compound_event = self._sut(events=[sine_event, sine_event, noise_event])

        # Assert
        self.assertEqual(1000, compound_event.fs)
        self.assertEqual(3, len(compound_event.events))
        self.assertEqual(20, compound_event.duration)
        self.assertEqual(20, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)

    def test_construct_from_offset_list_of_3_aligned_ms_and_pts(self):
        # Arrange
        sine_event_1 = SineComponent(start=200, duration=1000)
        sine_event_2 = SineComponent(start=300, duration=1000)
        sine_event_3 = SineComponent(start=500, duration=1000)

        # Act
        compound_event = CompoundComponent(events=[sine_event_1, sine_event_2, sine_event_3])

        # Assert
        self.assertEqual(1000, compound_event.fs)
        self.assertEqual(3, len(compound_event.events))
        self.assertEqual(1300, compound_event.duration)
        self.assertEqual(1300, compound_event.duration_pts)
        self.assertEqual(200, compound_event.start)
        self.assertEqual(200, compound_event.start_pts)

        sine_event_1.plot()
        sine_event_2.plot()
        sine_event_3.plot()
        compound_event.plot(show=SHOW)
        compound_event.plot_subplots(show=SHOW)

    def test_construct_from_offset_list_of_3_with_varied_fs(self):
        # Arrange
        sine_event_1 = SineComponent(start=200, duration=1000, fs=150)
        sine_event_2 = SineComponent(start=300, duration=1000, fs=150)
        sine_event_3 = SineComponent(start=500, duration=1000, fs=150)

        # Act
        compound_event = self._sut(events=[sine_event_1, sine_event_2, sine_event_3])

        # Assert
        self.assertEqual(150, compound_event.fs)
        self.assertEqual(3, len(compound_event.events))
        self.assertEqual(1300, compound_event.duration)
        self.assertEqual(195, compound_event.duration_pts)
        self.assertEqual(200, compound_event.start)
        self.assertEqual(30, compound_event.start_pts)

        # TODO: Finish
        sine_event_1.plot()
        sine_event_2.plot()
        sine_event_3.plot()
        compound_event.plot(show=SHOW)
        compound_event.plot_subplots(show=SHOW)

    def test_construct_from_offset_list_of_3_adjusted_start(self):
        # Arrange
        sine_event_1 = SineComponent(start=200, duration=1000)
        sine_event_2 = SineComponent(start=300, duration=1000)
        sine_event_3 = SineComponent(start=500, duration=1000)

        # Act
        compound_event = self._sut(events=[sine_event_1, sine_event_2, sine_event_3],
                                   start=0)

        # Assert
        self.assertEqual(1000, compound_event.fs)
        self.assertEqual(3, len(compound_event.events))
        self.assertEqual(1300, compound_event.duration)
        self.assertEqual(1300, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)

        # TODO: Finish
        sine_event_1.plot()
        sine_event_2.plot()
        sine_event_3.plot()
        compound_event.plot(show=SHOW)
        compound_event.plot_subplots(show=SHOW)

    def test_incompatible_events_fs_raises_error(self):
        # Arrange
        sine_event = SineComponent()
        noise_event = NoiseComponent(fs=100)

        # Acert
        self.assertRaises(ValueError, lambda: self._sut(events=[sine_event, sine_event, noise_event]))

    def test_create_from_fully_overlapping_long_list(self):
        # Arrange
        rng = np.random.RandomState(123)
        ev_kwargs = {'fs': 2000, 'duration': 100, 'envelope': partial(CosRiseEnvelope, rise=10)}
        n = 40
        evs = [SineComponent(freq=e_i, **ev_kwargs) for e_i in rng.randint(5, 10, n)]

        # Act
        compound_event = self._sut(evs)

        # Assert
        self.assertEqual(200, compound_event.y.shape[0])
        self.assertEqual(n, compound_event.channels().shape[0])
        self.assertEqual(200, compound_event.channels().shape[1])
        self.assertEqual(2000, compound_event.fs)
        self.assertEqual(n, len(compound_event.events))
        self.assertEqual(100, compound_event.duration)
        self.assertEqual(200, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)

        compound_event.plot(channels=True,
                            show=SHOW)

    def test_create_from_non_overlapping_long_list(self):
        # Arrange
        rng = np.random.RandomState(123)
        ev_kwargs = {'fs': 2000, 'duration': 100}
        n = 6
        evs = [SineComponent(freq=f,
                             start=s_i * 100,
                             **ev_kwargs) for s_i, f in enumerate(rng.randint(5, 10, 6))]

        # Act
        compound_event = self._sut(evs)

        # Assert
        self.assertEqual(200 * n, compound_event.y.shape[0])
        self.assertEqual(n, compound_event.channels().shape[0])
        self.assertEqual(200 * n, compound_event.channels().shape[1])
        self.assertEqual(2000, compound_event.fs)
        self.assertEqual(n, len(compound_event.events))
        self.assertEqual(100 * n, compound_event.duration)
        self.assertEqual(200 * n, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)

        compound_event.plot(channels=True,
                            show=SHOW)

    def test_create_from_partially_overlapping_long_list(self):
        # Arrange
        ev_kwargs = {'fs': 2000, 'duration': 100}
        n = 3
        start_step = 80
        evs = [SineComponent(freq=10,
                             start=s_i * start_step,
                             **ev_kwargs) for s_i in range(n)]

        # Act
        compound_event = self._sut(evs)

        # Assert
        self.assertEqual(n, compound_event.channels().shape[0])
        self.assertEqual(200 + 200 + 120, compound_event.channels().shape[1])
        self.assertEqual(2000, compound_event.fs)
        self.assertEqual(n, len(compound_event.events))
        self.assertEqual(260, compound_event.duration)
        self.assertEqual(520, compound_event.duration_pts)
        self.assertEqual(0, compound_event.start)
        self.assertEqual(0, compound_event.start_pts)

        compound_event.plot(channels=True,
                            show=SHOW)

    def test_eval_repr_equality(self):
        # Arrange
        ev_kwargs = {'fs': 2000,
                     'duration': 100}
        n = 3
        start_step = 80
        evs = [SineComponent(freq=10, start=s_i * start_step, **ev_kwargs) for s_i in range(n)]
        compound_event = CompoundComponent(evs)

        # Act
        clone = eval(compound_event.__repr__())

        # Assert
        self.assertEqual(compound_event, clone)
        self.assertEqual(clone.channels().shape[0], compound_event.channels().shape[0])
        self.assertEqual(clone.channels().shape[1], compound_event.channels().shape[1])
        self.assertEqual(clone.fs, compound_event.fs)
        self.assertEqual(len(clone.events), len(compound_event.events))
        self.assertListEqual(clone.events, compound_event.events)
        self.assertEqual(clone.duration, compound_event.duration)
        self.assertEqual(clone.duration_pts, compound_event.duration_pts)
        self.assertEqual(clone.start, compound_event.start)
        self.assertEqual(clone.start_pts, compound_event.start_pts)

        compound_event.plot(channels=True,
                            show=SHOW)

    def test_previous_start_bug(self):
        """
        This stim raised error when trying to add second event due to subtracting self.start rather than
        self.start_pts in CompoundComponent.channels. Make sure it doesn't come back...
        """
        # Arrange
        ev_kwargs = {'fs': 1600, 'duration': 200}
        noise_events = [NoiseComponent(mag=0.1, start=100, **ev_kwargs),
                        NoiseComponent(mag=0.06, start=300, **ev_kwargs)]

        # Act
        compound_event = self._sut(noise_events)

        compound_event.plot_subplots(show=SHOW)

        # Assert
        try:
            _ = compound_event.y  # Previous error
        except IndexError:
            self.fail("argh, bug.")

        self.assertEqual(2, compound_event.channels().shape[0])
        self.assertEqual(0.4 * 1600, compound_event.channels().shape[1])
        self.assertEqual(1600, compound_event.fs)
        self.assertEqual(2, len(compound_event.events))
        self.assertEqual(500 - 100, compound_event.duration)
        self.assertEqual(0.4 * 1600, compound_event.duration_pts)
        self.assertEqual(100, compound_event.start)
        self.assertEqual(160, compound_event.start_pts)

    def test_normalise_limits_range(self):
        # Arrange
        ev_kwargs = {'fs': 1600, 'duration': 200}
        noise_events = [NoiseComponent(mag=0.1, start=100, **ev_kwargs),
                        NoiseComponent(mag=0.06, start=300, **ev_kwargs)]

        compound_1 = self._sut(noise_events, normalise=False)
        compound_2 = self._sut(noise_events, normalise=True)

        self.assertFalse(compound_1 == compound_2)
        self.assertFalse(np.all(compound_1.y == compound_2.y))
        self.assertLessEqual(compound_2.y.max(), 1.0)
        self.assertGreaterEqual(compound_2.y.min(), 0.0)

    def test_cache_keeps_generated_array(self):
        # Arrange
        ev_kwargs = {'fs': 1600, 'duration': 200}
        noise_events = [NoiseComponent(mag=0.1, start=100, **ev_kwargs),
                        NoiseComponent(mag=0.06, start=300, **ev_kwargs)]

        compound_1 = self._sut(noise_events, cache=False)
        compound_2 = self._sut(noise_events, cache=True)

        # Act
        ev1_y = compound_1.y
        ev2_y = compound_2.y

        # Assert
        self.assertIsInstance(ev1_y, np.ndarray)
        self.assertIsInstance(ev2_y, np.ndarray)
        self.assertIsNone(compound_1._y)
        self.assertIsNotNone(compound_2._y)
